@@ -1,5 +1,5 @@
 package File::Dir::Dumper::Scanner;
-$File::Dir::Dumper::Scanner::VERSION = 'v0.2.3';
+$File::Dir::Dumper::Scanner::VERSION = 'v0.4.0';
 use warnings;
 use strict;
 use autodie;
@@ -18,6 +18,7 @@ use List::Util qw(min);
 
 use Class::XSAccessor
     accessors => {
+        _digest_cache => '_digest_cache',
         _digests => '_digests',
         _file_find => '_file_find',
         _group_cache => '_group_cache',
@@ -29,42 +30,6 @@ use Class::XSAccessor
     };
 
 use Digest ();
-=head1 NAME
-
-File::Dir::Dumper::Scanner - scans a directory and returns a stream of Perl
-hash-refs
-
-=head1 VERSION
-
-version v0.2.3
-
-=head1 SYNOPSIS
-
-    use File::Dir::Dumper::Scanner;
-
-    my $scanner = File::Dir::Dumper::Scanner->new(
-        {
-            dir => $dir_pathname
-        }
-    );
-
-    while (defined(my $token = $scanner->fetch()))
-    {
-    }
-
-=head1 METHODS
-
-=head2 $self->new({ dir => $dir_path, digests => [LIST]})
-
-Scans the directory $dir_path with the L<Digest>'s digests as contained
-in the list of strings pointed by the digests array reference. C<digests>
-is optional.
-
-=head2 my $hash_ref = $self->fetch()
-
-Outputs the next token as a hash ref.
-
-=cut
 
 sub _init
 {
@@ -104,6 +69,25 @@ sub _init
         }
         $self->_digests([sort {$a cmp $b} keys%$digests]);
     }
+    my $base = ($args->{digest_cache} || 'Dummy');
+    if ($base !~ /\A[A-Za-z_][A-Za-z_0-9]*\z/)
+    {
+        Carp::confess( "Invalid digest_cache format." );
+    }
+    my $cl = "File::Dir::Dumper::DigestCache::$base";
+    eval "require $cl";
+    if ($@)
+    {
+        die $@;
+    }
+    $self->_digest_cache(
+        scalar $cl->new(
+            {
+                params => ($args->{digest_cache_params} || +{}),
+
+            }
+        )
+    );
 
     $self->_user_cache({});
     $self->_group_cache({});
@@ -236,7 +220,7 @@ sub _get_group_name
 
 sub _calc_file_digests_key
 {
-    my $self = shift;
+    my ($self, $stat) = @_;
 
     my $digests = $self->_digests;
 
@@ -246,17 +230,27 @@ sub _calc_file_digests_key
     }
     my $result = $self->_result();
     my $path = $result->path;
-    my %ret;
-    foreach my $d (@$digests)
-    {
-        my $o = Digest->new($d);
-        open my $fh, '<', $path;
-        binmode $fh;
-        $o->addfile($fh);
-        $ret{$d} = $o->hexdigest;
-        close ($fh);
-    }
-    return [digests => \%ret];
+    my $ret = $self->_digest_cache->get_digests(
+        {
+            path => $result->full_components,
+            mtime => $stat->[9],
+            digests => $digests,
+            calc_cb => sub {
+                my %ret;
+                foreach my $d (@$digests)
+                {
+                    my $o = Digest->new($d);
+                    open my $fh, '<', $path;
+                    binmode $fh;
+                    $o->addfile($fh);
+                    $ret{$d} = $o->hexdigest;
+                    close ($fh);
+                }
+                return \%ret;
+            },
+        }
+    );
+    return [digests => $ret,];
 }
 
 sub _calc_file_or_dir_token
@@ -280,7 +274,7 @@ sub _calc_file_or_dir_token
             : (
                 type => "file",
                 size => $stat[7],
-                @{$self->_calc_file_digests_key()},
+                @{$self->_calc_file_digests_key(\@stat)},
             )
         ),
     };
@@ -321,6 +315,54 @@ sub _populate_queue
     $self->_last_result($self->_result());
 }
 
+
+1; # End of File::Dir::Dumper
+
+__END__
+
+=pod
+
+=encoding UTF-8
+
+=head1 NAME
+
+File::Dir::Dumper::Scanner - scans a directory and returns a stream of Perl
+hash-refs
+
+=head1 VERSION
+
+version v0.4.0
+
+=head1 SYNOPSIS
+
+    use File::Dir::Dumper::Scanner;
+
+    my $scanner = File::Dir::Dumper::Scanner->new(
+        {
+            dir => $dir_pathname
+        }
+    );
+
+    while (defined(my $token = $scanner->fetch()))
+    {
+    }
+
+=head1 VERSION
+
+version v0.4.0
+
+=head1 METHODS
+
+=head2 $self->new({ dir => $dir_path, digests => [LIST]})
+
+Scans the directory $dir_path with the L<Digest>'s digests as contained
+in the list of strings pointed by the digests array reference. C<digests>
+is optional.
+
+=head2 my $hash_ref = $self->fetch()
+
+Outputs the next token as a hash ref.
+
 =head1 AUTHOR
 
 Shlomi Fish, C<< <shlomif@cpan.org> >>
@@ -331,16 +373,12 @@ Please report any bugs or feature requests to C<bug-file-dir-dumper at rt.cpan.o
 the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=File-Dir-Dumper>.  I will be notified, and then you'll
 automatically be notified of progress on your bug as I make changes.
 
-
-
-
 =head1 SUPPORT
 
 You can find documentation for this module with the perldoc command.
 
     perldoc File::Dir::Dumper
 
-
 You can also look for information at:
 
 =over 4
@@ -363,16 +401,148 @@ L<http://search.cpan.org/dist/File-Dir-Dumper>
 
 =back
 
-
 =head1 ACKNOWLEDGEMENTS
 
-
 =head1 COPYRIGHT & LICENSE
 
 Copyright 2008 Shlomi Fish, all rights reserved.
 
 This program is released under the following license: MIT/X11 Licence.
 
-=cut
+=head1 AUTHOR
 
-1; # End of File::Dir::Dumper
+Shlomi Fish <shlomif@cpan.org>
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is Copyright (c) 2008 by Shlomi Fish.
+
+This is free software, licensed under:
+
+  The MIT (X11) License
+
+=head1 BUGS
+
+Please report any bugs or feature requests on the bugtracker website
+L<https://github.com/shlomif/file-dir-dumper/issues>
+
+When submitting a bug or request, please include a test-file or a
+patch to an existing test-file that illustrates the bug or desired
+feature.
+
+=for :stopwords cpan testmatrix url annocpan anno bugtracker rt cpants kwalitee diff irc mailto metadata placeholders metacpan
+
+=head1 SUPPORT
+
+=head2 Perldoc
+
+You can find documentation for this module with the perldoc command.
+
+  perldoc File::Dir::Dumper
+
+=head2 Websites
+
+The following websites have more information about this module, and may be of help to you. As always,
+in addition to those websites please use your favorite search engine to discover more resources.
+
+=over 4
+
+=item *
+
+MetaCPAN
+
+A modern, open-source CPAN search engine, useful to view POD in HTML format.
+
+L<http://metacpan.org/release/File-Dir-Dumper>
+
+=item *
+
+Search CPAN
+
+The default CPAN search engine, useful to view POD in HTML format.
+
+L<http://search.cpan.org/dist/File-Dir-Dumper>
+
+=item *
+
+RT: CPAN's Bug Tracker
+
+The RT ( Request Tracker ) website is the default bug/issue tracking system for CPAN.
+
+L<https://rt.cpan.org/Public/Dist/Display.html?Name=File-Dir-Dumper>
+
+=item *
+
+AnnoCPAN
+
+The AnnoCPAN is a website that allows community annotations of Perl module documentation.
+
+L<http://annocpan.org/dist/File-Dir-Dumper>
+
+=item *
+
+CPAN Ratings
+
+The CPAN Ratings is a website that allows community ratings and reviews of Perl modules.
+
+L<http://cpanratings.perl.org/d/File-Dir-Dumper>
+
+=item *
+
+CPAN Forum
+
+The CPAN Forum is a web forum for discussing Perl modules.
+
+L<http://cpanforum.com/dist/File-Dir-Dumper>
+
+=item *
+
+CPANTS
+
+The CPANTS is a website that analyzes the Kwalitee ( code metrics ) of a distribution.
+
+L<http://cpants.cpanauthors.org/dist/File-Dir-Dumper>
+
+=item *
+
+CPAN Testers
+
+The CPAN Testers is a network of smokers who run automated tests on uploaded CPAN distributions.
+
+L<http://www.cpantesters.org/distro/F/File-Dir-Dumper>
+
+=item *
+
+CPAN Testers Matrix
+
+The CPAN Testers Matrix is a website that provides a visual overview of the test results for a distribution on various Perls/platforms.
+
+L<http://matrix.cpantesters.org/?dist=File-Dir-Dumper>
+
+=item *
+
+CPAN Testers Dependencies
+
+The CPAN Testers Dependencies is a website that shows a chart of the test results of all dependencies for a distribution.
+
+L<http://deps.cpantesters.org/?module=File::Dir::Dumper>
+
+=back
+
+=head2 Bugs / Feature Requests
+
+Please report any bugs or feature requests by email to C<bug-file-dir-dumper at rt.cpan.org>, or through
+the web interface at L<https://rt.cpan.org/Public/Bug/Report.html?Queue=File-Dir-Dumper>. You will be automatically notified of any
+progress on the request by the system.
+
+=head2 Source Code
+
+The code is open to the world, and available for you to hack on. Please feel free to browse it and play
+with it, or whatever. If you want to contribute patches, please send me a diff or prod me to pull
+from your repository :)
+
+L<https://github.com/shlomif/perl-File-Dir-Dumper>
+
+  git clone https://github.com/shlomif/perl-File-Dir-Dumper
+
+=cut
